/* elf.h - Extended Load Format header */

/* Copyright 1995-1996 Wind River Systems, Inc. */

/*
modification history
--------------------
01b,16feb96,pad  cosmetic changes in comments after code review.
01a,07dec95,pad  adapted to Tornado from version 01f of VxWorks elf.h.
*/

#ifndef __INCelfh
#define __INCelfh

#ifdef __cplusplus
extern "C" {
#endif

/*
 *	Copyright (c) 1990 UNIX System Laboratories, Inc.
 *	Copyright (c) 1984, 1986, 1987, 1988, 1989, 1990 AT&T
 *	  All Rights Reserved
 *
 *	THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF
 *	UNIX System Laboratories, Inc.
 *	The copyright notice above does not evidence any
 *	actual or intended publication of such source code.
 */

/* ELF types definition */

typedef unsigned long   Elf32_Addr;	/* unsigned program address */
typedef unsigned short  Elf32_Half;	/* unsigned medium integer */
typedef unsigned long   Elf32_Off;	/* unsigned file offset */
typedef long            Elf32_Sword;	/* signed large integer */
typedef unsigned long   Elf32_Word;	/* unsigned large integer */

#define ELF32_FSZ_ADDR	4
#define ELF32_FSZ_HALF	2
#define ELF32_FSZ_OFF	4
#define ELF32_FSZ_SWORD	4
#define ELF32_FSZ_WORD	4


/* ELF header */

#define EI_NIDENT	16		/* identification size */

typedef struct {
	unsigned char	e_ident[EI_NIDENT];	/* identification bytes */
	Elf32_Half	e_type;			/* file type */
	Elf32_Half	e_machine;		/* target machine */
	Elf32_Word	e_version;		/* file version */
	Elf32_Addr	e_entry;		/* start address */
	Elf32_Off	e_phoff;		/* program hdrs file offset */
	Elf32_Off	e_shoff;		/* section hdrs file offset */
	Elf32_Word	e_flags;		/* processor specific flags */
	Elf32_Half	e_ehsize;		/* ELF header size */
	Elf32_Half	e_phentsize;		/* program header size */
	Elf32_Half	e_phnum;		/* number of program headers */
	Elf32_Half	e_shentsize;		/* section header size */
	Elf32_Half	e_shnum;		/* number of section section */
	Elf32_Half	e_shstrndx;		/* idx of section string tbl */
} Elf32_Ehdr;

/* Identification indexes */

#define EI_MAG0		0		/* file identification */
#define EI_MAG1		1		/* file identification */
#define EI_MAG2		2		/* file identification */
#define EI_MAG3		3		/* file identification */
#define EI_CLASS	4		/* file class */
#define EI_DATA		5		/* data encoding */
#define EI_VERSION	6		/* file version */
#define EI_PAD		7		/* start of padding bytes */

/* Magic ELF identifier definitions */

#define ELFMAG0		0x7f		/* at EI_MAG0 */
#define ELFMAG1		'E'		/* at EI_MAG1 */
#define ELFMAG2		'L'		/* at EI_MAG2 */
#define ELFMAG3		'F'		/* at EI_MAG3 */
#define ELFMAG		"\177ELF"	/* magic ELF identifier */
#define SELFMAG		4		/* size of magic ELF identifier */

/* File class definitions */

#define ELFCLASSNONE	0		/* invalid class */
#define ELFCLASS32	1		/* 32-bit objects */
#define ELFCLASS64	2		/* 64-bit objects */
#define ELFCLASSNUM	3

/* Data encoding */

#define ELFDATANONE	0		/* invalid encoding */
#define ELFDATA2LSB	1		/* little-endian encoding */
#define ELFDATA2MSB	2		/* big-endian encoding */
#define ELFDATANUM	3

/* File type definitions */

#define ET_NONE		0		/* no file type */
#define ET_REL		1		/* relocatable file */
#define ET_EXEC		2		/* executable file */
#define ET_DYN		3		/* shared object file */
#define ET_CORE		4		/* core file */
#define ET_NUM		5
#define	ET_LOPROC	0xff00		/* low bound processor specific range */
#define	ET_HIPROC	0xffff		/* ...high bound */

/* Target machine definitions */

#define EM_NONE		0		/* no machine */
#define EM_M32		1		/* AT&T WE 32100 */
#define EM_SPARC	2		/* Sun SPARC */
#define EM_386		3		/* Intel 80386 */
#define EM_68K		4		/* Motorola 68000 */
#define EM_88K		5		/* Motorola 88000 */
#define EM_486		6		/* Intel 80486 */
#define EM_860		7		/* Intel i860 */
#define EM_MIPS		8		/* MIPS family  */
#define EM_PPC_OLD	17		/* PowerPC family - EABI draft 1.0 */
#define EM_960		19		/* Intel i960 family */
#define EM_PPC		20		/* PowerPC family */

/* Version definitions */

#define EV_NONE		0		/* invalid version */
#define EV_CURRENT	1		/* current version */
#define EV_NUM		2


/* Program header */

typedef struct {
	Elf32_Word	p_type;		/* entry type */
	Elf32_Off	p_offset;	/* file offset */
	Elf32_Addr	p_vaddr;	/* virtual address */
	Elf32_Addr	p_paddr;	/* physical address */
	Elf32_Word	p_filesz;	/* segment size in file */
	Elf32_Word	p_memsz;	/* segment size in memory */
	Elf32_Word	p_flags;	/* segment flags */
	Elf32_Word	p_align;	/* memory/file alignment */
} Elf32_Phdr;

/* Segment type definitions */

#define PT_NULL		0		/* unused entry */
#define PT_LOAD		1		/* loadable segment */
#define PT_DYNAMIC	2		/* dynamic linking information */
#define PT_INTERP	3		/* interpreter information */
#define PT_NOTE		4		/* vendor specific information */
#define PT_SHLIB	5		/* reserved */
#define PT_PHDR		6		/* program header table information */
#define PT_NUM		7
#define PT_LOPROC	0x70000000	/* low bound processor specific range */
#define PT_HIPROC	0x7fffffff	/* ...high bound */

/* Segment flag definitions */

#define PF_X		0x1		/* executable segment */
#define PF_W		0x2		/* writable segment */
#define PF_R		0x4		/* readable segment */
#define PF_MASKPROC	0xf0000000	/* processor specific values */


/* Section header */

typedef struct {
	Elf32_Word	sh_name;	/* section name */
	Elf32_Word	sh_type;	/* section type */
	Elf32_Word	sh_flags;	/* section flag */
	Elf32_Addr	sh_addr;	/* virtual address */
	Elf32_Off	sh_offset;	/* file offset */
	Elf32_Word	sh_size;	/* section size */
	Elf32_Word	sh_link;	/* misc info */
	Elf32_Word	sh_info;	/* misc info */
	Elf32_Word	sh_addralign;	/* memory alignment */
	Elf32_Word	sh_entsize;	/* entry size if table */
} Elf32_Shdr;

/* Section types */

#define SHT_NULL	0		/* inactive (no associated section) */
#define SHT_PROGBITS	1		/* program information */
#define SHT_SYMTAB	2		/* symbol table */
#define SHT_STRTAB	3		/* string table */
#define SHT_RELA	4		/* relocation with addend */
#define SHT_HASH	5		/* symbol hash table */
#define SHT_DYNAMIC	6		/* dynamic linking information */
#define SHT_NOTE	7		/* vendor specific information */
#define SHT_NOBITS	8		/* program information not allocated */
#define SHT_REL		9		/* relocation without addend */
#define SHT_SHLIB	10		/* reserved */
#define SHT_DYNSYM	11		/* minimal symbol table */
#define SHT_NUM		12		/* reserved */
#define SHT_LOUSER	0x80000000	/* low bound application specif range */
#define SHT_HIUSER	0xffffffff	/* ...high bound */
#define	SHT_LOPROC	0x70000000	/* low bound processor specific range */
#define	SHT_HIPROC	0x7fffffff	/* ...high bound */

/* Section flags */

#define SHF_WRITE	0x1		/* writable data */
#define SHF_ALLOC	0x2		/* allocated data */
#define SHF_EXECINSTR	0x4		/* executable instructions */
#define SHF_MASKPROC	0xf0000000	/* processor specific values */

/* Reserved section header indeces */

#define SHN_UNDEF	0		/* undefined section */
#define SHN_LORESERVE	0xff00		/* lower bound of reserved indexes */
#define SHN_LOPROC	0xff00		/* low bound processor specific range */
#define SHN_HIPROC	0xff1f		/* ...high bound */
#define SHN_ABS		0xfff1		/* absolute value */
#define SHN_COMMON	0xfff2		/* common symbols */
#define SHN_HIRESERVE	0xffff		/* higher bound of reserved indexes */


/* Symbol table */

typedef struct {
	Elf32_Word	st_name;	/* idx of symbol name in string table */
	Elf32_Addr	st_value;	/* symbol value */
	Elf32_Word	st_size;	/* symbol size */
	unsigned char	st_info;	/* symbol type and binding attribute */
	unsigned char	st_other;	/* reserved */
	Elf32_Half	st_shndx;	/* associated section header index */
} Elf32_Sym;

#define STN_UNDEF	0	/* initial entry index in symbol table */

/*
 * The macros compose and decompose values for the st_info field
 *
 *	bind = ELF32_ST_BIND(S.st_info)
 *	type = ELF32_ST_TYPE(S.st_info)
 *	S.st_info = ELF32_ST_INFO(bind, type)
 */

#define ELF32_ST_BIND(info)		((info) >> 4)
#define ELF32_ST_TYPE(info)		((info) & 0xf)
#define ELF32_ST_INFO(bind,type)	(((bind)<<4)+((type)&0xf))

/* Symbol binding definitions */

#define STB_LOCAL	0		/* local symbol */
#define STB_GLOBAL	1		/* global symbol */
#define STB_WEAK	2		/* weak symbol */
#define STB_NUM		3
#define STB_LOPROC	13		/* low bound processor specific range */
#define STB_HIPROC	15		/* ...high bound */

/* Symbol type definitions */

#define STT_NOTYPE	0		/* no specified type */
#define STT_OBJECT	1		/* data object (variables...) */
#define STT_FUNC	2		/* function */
#define STT_SECTION	3		/* section */
#define STT_FILE	4		/* source file */
#define STT_NUM		5
#define STT_LOPROC	13		/* low bound processor specific range */
#define STT_HIPROC	15		/* ...high bound */


/* Relocation information */

typedef struct {
	Elf32_Addr	r_offset;	/* offset from beginning of section */
	Elf32_Word	r_info;		/* symbol index and relocation type */
} Elf32_Rel;

typedef struct {
	Elf32_Addr	r_offset;	/* offset from beginning of section */
	Elf32_Word	r_info;		/* symbol index and relocation type */
	Elf32_Sword	r_addend;	/* constant addend */
} Elf32_Rela;

/*
 * The macros compose and decompose values for Rel.r_info, Rela.f_info
 *
 *	sym = ELF32_R_SYM(R.r_info)
 *	type = ELF32_R_TYPE(R.r_info)
 *	R.r_info = ELF32_R_INFO(sym, type)
 */

#define ELF32_R_SYM(info)	((info)>>8)
#define ELF32_R_TYPE(info)	((unsigned char)(info))
#define ELF32_R_INFO(sym,type)	(((sym)<<8)+(unsigned char)(type))

#ifdef __cplusplus
}
#endif

#endif /* __INCelfh */
